from rosetta import *
init()


def interface_ddg(pose, resid, mut_res):

	dock_jump = 1
	wt_p = Pose()
	mut_p = Pose()

	wt_p.assign(pose)
	mut_p.assign(pose)
	mutate_residue(mut_p, resid, mut_res)

	scorefxn = ScoreFunction()
	scorefxn.set_weight(fa_atr, 0.44)
	scorefxn.set_weight(fa_rep, 0.07)
	scorefxn.set_weight(fa_sol, 1.0)
	scorefxn.set_weight(hbond_bb_sc, 0.5)
	scorefxn.set_weight(hbond_sc, 1.0)

	dock_prot = DockingProtocol(dock_jump)
	dock_prot.set_highres_scorefxn(scorefxn)

	#setting up packer options
	tf = standard_task_factory()
	tf.push_back(RestrictToRepacking())
	prevent_repacking = PreventRepacking()

	for i in range(1, pose.total_residue()+1):
		rsd1 = pose.residue(resid)
		rsd2 = pose.residue(i)
		if (rsd1.nbr_atom_xyz().distance_squared(rsd2.nbr_atom_xyz())>64.0):
			prevent_repacking.include_residue(i)	

	tf.push_back(prevent_repacking)

	#setup packer	
	pack_mover = PackRotamersMover(scorefxn)
	pack_mover.task_factory(tf)

	#applying side chain packer to wt and mutant
	pack_mover.apply(wt_p)
	pack_mover.apply(mut_p)

	wt_score = dock_prot.calc_interaction_energy(wt_p)
	mut_score = dock_prot.calc_interaction_energy(mut_p)
	ddg = mut_score - wt_score

	pdbname = p.pdb_info().chain(resid)+str(p.pdb_info().number(resid))+wt_p.sequence()[resid-1]+"to"
	#dump_pdb(wt_p,pdbname+ wt_p.sequence()[resid-1]+".pdb")
	#dump_pdb(mut_p,pdbname+ mut_p.sequence()[resid-1]+".pdb")
	return ddg

p = Pose()
pose_from_pdb(p, "aacap1_0290.pdb")
starting_p = Pose()

#parameters
dock_jump = 1
interface_dist = 8.0 #angstroms
DockingProtocol().setup_foldtree(p)

scorefxn = create_score_function('standard')
scorefxn(p) #needed for proper Interface calculation

interface = Interface(dock_jump)
interface.distance(8.0)

interface.calculate(p)

#store starting structure
starting_p.assign(p)

ddg_scorefxn = ScoreFunction()
ddg_scorefxn.set_weight(fa_atr, 0.44)
ddg_scorefxn.set_weight(fa_rep, 0.07)
ddg_scorefxn.set_weight(fa_sol, 1.0)
ddg_scorefxn.set_weight(hbond_bb_sc, 0.5)
ddg_scorefxn.set_weight(hbond_sc, 1.0)

for i in range(1, p.total_residue()+1):
	if (interface.is_interface(i) == True):
		p.assign(starting_p)
		ddg = interface_ddg(p, i, 'A')
		f = open("ala_scan_output.txt", 'a')
		mutname = p.pdb_info().chain(i)+str(p.pdb_info().number(i))+p.sequence()[i-1]+"to"+'A'+".pdb" 
		f.write( mutname + "  ddG: " + str(round(ddg, 3)) +'\n')

